/**
 * @file js tools
 * @name canfei.js
 * @author Wang Jianhui <wangjianhui@herebookstore.com> (https://github.com/canfeit/canfei/blob/master/README.md)
 * @copyright canfeit 2017
 * @version 1.0.0
 */

/**
 *  数据类型判断
 * @param {any} o
 * @return {string}
 * @example
 * // returns 'Undefined'
 * ComplexType(undefined)
 * typeof o === 'undefined'
 * isNaN(o)
 * @example
 * // returns 'Null'
 * ComplexType(null)
 * o === null
 * @example
 * // returns 'Array'
 * ComplexType([])
 * Array.isArray([])
 * @example
 * // returns 'String'
 * ComplexType('')
 * typeof '' === 'string'
 * @example
 * // returns 'Number'
 * ComplexType(1)
 * typeof 1 === 'number'
 * @example
 * // returns 'Boolean'
 * ComplexType(true)
 * typeof true === 'boolean'
 * @example
 * // returns 'Function'
 * ComplexType(function(){})
 * typeof function(){} === 'function'
 * @example
 * // returns 'Object'
 * ComplexType({})
 * @example
 * // returns 'Date'
 * ComplexType(new Date())
 * @example
 * // returns 'RegExp'
 * ComplexType(/./)
 */
const ComplexType = function (o) {
    return Object.prototype.toString.call(o).slice(8, -1)
}
/**
 * 判断非空对象
 * @param {any} o
 * @returns
 */
const isObjects = function (o) { // 非空对象(或数组)
    for (let k in o) { return true }
    return false
}
/**
 * 判断非空数组
 * @param {any} o
 * @returns
 */
const isArrays = function (o) { // 非空数组
    return Array.isArray(o) && o.length
}
/**
 * 判断无效数据
 * 假值:undefined,null,0,+0,-0,NaN,'',"",false
 * 空对象:{}
 * 空数组:[]
 * @param {any} o
 * @returns
 */
const isFalse = function (o) {
    return !o ||// 假值
        (typeof o === 'object' && (
            (ComplexType(o) === 'String' && o.toString() === '') ||
            (ComplexType(o) === 'Number' && o.toString() === '0') ||
            (ComplexType(o) === 'Boolean' && o.toString() === 'false') ||
            ((ComplexType(o) === 'Array' || ComplexType(o) === 'Object') &&
                !isObjects(o))// 空对象(或数组)
        ))
}

if (typeof exports === 'object' &&
    typeof module !== 'undefined' &&
    !module.nodeType &&
    module.exports) {
    module.exports = {
        ComplexType,
        isObjects,
        isArrays,
        isFalse
    }
}
